import logging
from typing import Optional, Type, Callable, Iterator
from ir_datasets.indices import PickleLz4FullStore
from datamaestro.download import Download
from datamaestro.utils import FileChecker
from pathlib import Path
import urllib3


class lz4docstore_downloader(Download):
    """Uses ir_datasets Lz4FullStore to build a document store for a stream of documents"""

    def __init__(
        self,
        varname: str,
        url: str,
        iter_factory: Callable[[Path], Iterator],
        doc_cls: Type,
        lookup_field: str,
        *,
        count_hint: Optional[int] = None,
        size: Optional[int] = None,
        checker: FileChecker = None,
    ):
        super().__init__(varname)
        self.iter_factory = iter_factory
        self.url = url
        self.doc_cls = doc_cls
        self.size = size
        self.lookup_field = lookup_field
        self.count_hint = count_hint
        self.checker = checker

        p = urllib3.util.parse_url(self.url)
        assert p is not None
        self.name = Path(p.path).with_suffix("").name

    def prepare(self):
        return self.definition.datapath / self.name

    def download(self, force=False):
        # Creates directory if needed
        destination = self.definition.datapath / self.name
        destination.mkdir(exist_ok=True)

        # Early exit
        if (destination / "done").is_file() and not force:
            return True

        # Download (cache)
        logging.info("Building the document index")
        with self.context.downloadURL(self.url, size=self.size) as file:
            # Checks the file
            if self.checker:
                self.checker.check(file.path)

            # Builds the LZ4 store
            store = PickleLz4FullStore(
                destination,
                lambda: self.iter_factory(Path(file.path)),
                self.doc_cls,
                lookup_field=self.lookup_field,
                index_fields=[self.lookup_field],
                key_field_prefix=None,
                size_hint=None,
                count_hint=self.count_hint,
            )
            store.build()

            # All good!
            (destination / "done").touch()



from abc import ABC, abstractmethod
from functools import partial
import logging
from pathlib import Path
from typing import Any, Iterator, NamedTuple, Tuple, Type, List
from attr import define
import attrs
import ir_datasets
from ir_datasets.indices import PickleLz4FullStore
from ir_datasets.formats import (
    GenericDoc,
    GenericQuery,
    GenericDocPair,
    TrecParsedDoc,
    TrecQuery,
)
import ir_datasets.datasets as _irds
from experimaestro import Config, Param
from experimaestro.compat import cached_property
from experimaestro import Option
import datamaestro_text.data.ir as ir
from datamaestro_text.data.ir.base import (
    Record,
    TopicRecord,
    DocumentRecord,
    SimpleTextItem,
    AdhocAssessedTopic,
    SimpleAdhocAssessment,
    IDItem,
    IDTopicRecord,
    IDDocumentRecord,
)
import datamaestro_text.data.ir.formats as formats


# Interface between ir_datasets and datamaestro:
# provides adapted data types


class IRDSId(Config):
    irds: Option[str]
    """The id to load the dataset from ir_datasets"""

    @classmethod
    def __xpmid__(cls):
        return f"ir_datasets.{cls.__qualname__}"

    @cached_property
    def dataset(self):
        return ir_datasets.load(self.irds)

    def iter(self) -> Iterator[Record]:
        """Returns an iterator over topics"""
        for query in self.dataset.queries_iter():
            yield self.factory(query)

    def count(self):
        return self.dataset.queries_count()


class AdhocAssessments(ir.AdhocAssessments, IRDSId):
    def iter(self):
        """Returns an iterator over assessments"""
        ds = self.dataset

        class Qrels(dict):
            def __missing__(self, key):
                qqrel = AdhocAssessedTopic(key, [])
                self[key] = qqrel
                return qqrel

        qrels = Qrels()
        for qrel in ds.qrels_iter():
            qrels[qrel.query_id].assessments.append(
                SimpleAdhocAssessment(qrel.doc_id, qrel.relevance)
            )

        return qrels.values()


class tuple_constructor:
    def __init__(self, target_cls: Type, id_field: str, *fields: str):
        self.id_field = id_field
        self.target_cls = target_cls
        self.fields = fields

    def check(self, source_cls: Type):
        source_fields = tuple(f for f in source_cls._fields if f != self.id_field)
        assert source_fields == self.fields, (
            "Internal error: Fields do not match, "
            f"source({source_cls.__qualname__})={source_fields}/{self.id_field} [vs] target={self.fields}"
        )

    def __call__(self, recordtype, entry):
        values = tuple(getattr(entry, key) for key in self.fields)
        return recordtype(
            IDItem(getattr(entry, self.id_field)), self.target_cls(*values)
        )


class Documents(ir.DocumentStore, IRDSId):
    CONVERTERS = {
        GenericDoc: tuple_constructor(SimpleTextItem, "doc_id", "text"),
        _irds.beir.BeirCordDoc: tuple_constructor(
            formats.CordDocument, "doc_id", "text", "title", "url", "pubmed_id"
        ),
        _irds.beir.BeirTitleDoc: tuple_constructor(
            formats.TitleDocument, "doc_id", "text", "title"
        ),
        _irds.beir.BeirTitleUrlDoc: tuple_constructor(
            formats.TitleUrlDocument, "doc_id", "text", "title", "url"
        ),
        _irds.msmarco_document.MsMarcoDocument: tuple_constructor(
            formats.MsMarcoDocument, "doc_id", "url", "title", "body"
        ),
        _irds.cord19.Cord19FullTextDoc: tuple_constructor(
            formats.CordFullTextDocument,
            "doc_id",
            "title",
            "doi",
            "date",
            "abstract",
            "body",
        ),
        _irds.nfcorpus.NfCorpusDoc: tuple_constructor(
            formats.NFCorpusDocument, "doc_id", "url", "title", "abstract"
        ),
        TrecParsedDoc: tuple_constructor(
            formats.TrecParsedDocument, "doc_id", "title", "body", "marked_up_doc"
        ),
        _irds.wapo.WapoDoc: tuple_constructor(
            formats.WapoDocument,
            "doc_id",
            "url",
            "title",
            "author",
            "published_date",
            "kicker",
            "body",
            "body_paras_html",
            "body_media",
        ),
        _irds.tweets2013_ia.TweetDoc: tuple_constructor(
            formats.TweetDoc,
            "doc_id",
            "text",
            "user_id",
            "created_at",
            "lang",
            "reply_doc_id",
            "retweet_doc_id",
            "source",
            "source_content_type",
        ),
    }

    """Wraps an ir datasets collection -- and provide a default text
    value depending on the collection itself"""

    # List of fields
    # self.dataset.docs_cls()._fields

    def iter(self) -> Iterator[ir.DocumentRecord]:
        """Returns an iterator over adhoc documents"""
        for doc in self.dataset.docs_iter():
            yield self.converter(self.document_recordtype, doc)

    @property
    def documentcount(self):
        return self.dataset.docs_count()

    @cached_property
    def store(self):
        return self.dataset.docs_store()

    @cached_property
    def _docs(self):
        return self.dataset.docs_iter()

    def docid_internal2external(self, ix: int):
        return self._docs[ix].doc_id

    def document_ext(self, docid: str) -> DocumentRecord:
        return self.converter(self.document_recordtype, self.store.get(docid))

    def documents_ext(self, docids: List[str]) -> DocumentRecord:
        """Returns documents given their external IDs (optimized for batch)"""
        retrieved = self.store.get_many(docids)
        return [
            self.converter(self.document_recordtype, retrieved[docid])
            for docid in docids
        ]

    def document_int(self, ix):
        return self.converter(self.document_recordtype, self._docs[ix])

    @cached_property
    def document_recordtype(self):
        return DocumentRecord.from_types(
            f"{self.converter.target_cls.__name__}Record",
            IDItem,
            self.converter.target_cls,
            module=__name__,
        )

    @cached_property
    def converter(self):
        converter = Documents.CONVERTERS[self.dataset.docs_cls()]
        converter.check(self.dataset.docs_cls())
        return converter


if hasattr(_irds, "miracl"):
    Documents.CONVERTERS[_irds.miracl.MiraclDoc] = tuple_constructor(
        formats.DocumentWithTitle, "doc_id", "title", "text"
    )


# Fix while PR https://github.com/allenai/ir_datasets/pull/252
# is not in.
class DMPickleLz4FullStore(PickleLz4FullStore):
    def get_many(self, doc_ids, field=None):
        result = {}
        field_idx = self._doc_cls._fields.index(field) if field is not None else None
        for doc in self.get_many_iter(doc_ids):
            if field is not None:
                result[getattr(doc, self._id_field)] = doc[field_idx]
            else:
                result[getattr(doc, self._id_field)] = doc
        return result


class LZ4DocumentStore(ir.DocumentStore):
    """A LZ4-based document store"""

    path: Param[Path]

    #: Lookup field
    lookup_field: Param[str]

    # Extra indexed fields (e.g. URLs)
    index_fields: List[str]

    @cached_property
    def store(self):
        return DMPickleLz4FullStore(
            self.path, None, self.data_cls, self.lookup_field, self.index_fields
        )

    @cached_property
    def _docs(self):
        return self.store.__iter__()

    def docid_internal2external(self, ix: int):
        return getattr(self._docs[ix], self.store._id_field)

    def document_ext(self, docid: str) -> DocumentRecord:
        return self.converter(self.document_recordtype, self.store.get(docid))

    def documents_ext(self, docids: List[str]) -> DocumentRecord:
        """Returns documents given their external IDs (optimized for batch)"""
        retrieved = self.store.get_many(docids)
        return [
            self.converter(self.document_recordtype, retrieved[docid])
            for docid in docids
        ]

    def converter(self, data):
        """Converts a document from LZ4 tuples to any other format"""
        # By default, use identity
        return data

    def iter(self) -> Iterator[DocumentRecord]:
        """Returns an iterator over documents"""
        return map(
            partial(self.converter, self.document_recordtype), self.store.__iter__()
        )

    def documentcount(self):
        if self.count:
            return self.count
        return self.store.count()


class TopicsHandler(ABC):
    @abstractmethod
    def topic_int(self, internal_topic_id: int) -> TopicRecord:
        """Returns a document given its internal ID"""
        ...

    @abstractmethod
    def topic_ext(self, external_topic_id: str) -> TopicRecord:
        """Returns a document given its external ID"""
        ...

    @abstractmethod
    def iter(self) -> Iterator[TopicRecord]:
        """Returns an iterator over topics"""
        ...


class SimpleTopicsHandler(TopicsHandler):
    def __init__(self, converter, topics: "Topics"):
        self.converter = converter
        self._topics = topics
        self.target_cls = converter.target_cls
        converter.check(topics.dataset.queries_cls())

    def topic_int(self, internal_topic_id: int) -> TopicRecord:
        """Returns a document given its internal ID"""
        return self.topics_list[internal_topic_id]

    def topic_ext(self, external_topic_id: int) -> TopicRecord:
        """Returns a document given its external ID"""
        return self.topics_map[external_topic_id]

    def iter(self) -> Iterator[TopicRecord]:
        """Returns an iterator over topics"""
        yield from self.topics_list

    @cached_property
    def topics_map(self):
        return self.topics[0]

    @cached_property
    def topics_list(self):
        return self.topics[1]

    @cached_property
    def topics(self):
        topic_map = {}
        topic_list = []
        for query in self._topics.dataset.queries_iter():
            record = self.converter(self._topics.topic_recordtype, query)
            topic_map[query.query_id] = record
            topic_list.append(record)

        return topic_map, topic_list


class Topics(ir.TopicsStore, IRDSId):
    CONVERTERS = {
        GenericQuery: tuple_constructor(SimpleTextItem, "query_id", "text"),
        _irds.beir.BeirCovidQuery: tuple_constructor(
            formats.TrecTopic, "query_id", "text", "query", "narrative"
        ),
        _irds.beir.BeirUrlQuery: tuple_constructor(
            formats.UrlTopic, "query_id", "text", "url"
        ),
        _irds.nfcorpus.NfCorpusQuery: tuple_constructor(
            formats.NFCorpusTopic, "query_id", "title", "all"
        ),
        TrecQuery: tuple_constructor(
            formats.TrecQuery, "query_id", "title", "description", "narrative"
        ),
        _irds.tweets2013_ia.TrecMb13Query: tuple_constructor(
            formats.TrecMb13Query, "query_id", "query", "time", "tweet_time"
        ),
        _irds.tweets2013_ia.TrecMb14Query: tuple_constructor(
            formats.TrecMb14Query,
            "query_id",
            "query",
            "time",
            "tweet_time",
            "description",
        ),
    }

    HANDLERS = {
        cls: partial(SimpleTopicsHandler, converter)
        for cls, converter in CONVERTERS.items()
    }

    def count(self):
        return self.dataset.queries_count()

    @cached_property
    def topic_recordtype(self) -> Type[Record]:
        return TopicRecord.from_types(
            f"{self.handler.target_cls.__name__}Record",
            IDItem,
            self.handler.target_cls,
            module=Topics.__class__.__module__,
        )

    @cached_property
    def handler(self):
        handler = Topics.HANDLERS[self.dataset.queries_cls()](self)
        return handler

    def topic_int(self, internal_topic_id: int) -> TopicRecord:
        """Returns a document given its internal ID"""
        return self.handler.topic_int(internal_topic_id)

    def topic_ext(self, external_topic_id: str) -> TopicRecord:
        """Returns a document given its external ID"""
        return self.handler.topic_ext(external_topic_id)

    def iter(self) -> Iterator[TopicRecord]:
        """Returns an iterator over topics"""
        return self.handler.iter()


if hasattr(_irds.trec_cast, "Cast2022Query"):
    from datamaestro_text.data.conversation.base import (
        ConversationTreeNode,
        DecontextualizedDictItem,
        ConversationHistory,
    )

    class CastTopicsHandler(TopicsHandler):
        def __init__(self, dataset):
            self.dataset = dataset

        @property
        @abstractmethod
        def records(self):
            ...

        @cached_property
        def ext2records(self):
            return {record.topic.get_id(): record for record in self.records}

        def topic_int(self, internal_topic_id: int) -> TopicRecord:
            """Returns a document given its internal ID"""
            return self.records[internal_topic_id]

        def topic_ext(self, external_topic_id: str) -> TopicRecord:
            """Returns a document given its external ID"""
            return self.ext2records[external_topic_id]

        def iter(self) -> Iterator[ir.TopicRecord]:
            """Returns an iterator over topics"""
            return iter(self.records)

    class Cast2020TopicsHandler(CastTopicsHandler):
        @cached_property
        def records(self):
            try:
                topic_number = None
                node = None
                conversation = []
                records = []

                class Cast2020TopicRecord(TopicRecord):
                    class_types = [
                        IDItem,
                        SimpleTextItem,
                        DecontextualizedDictItem,
                        ConversationHistory,
                    ]

                for (
                    query
                ) in self.dataset.queries_iter():  # type: _irds.trec_cast.Cast2020Query
                    decontextualized = DecontextualizedDictItem(
                        "manual",
                        {
                            "manual": query.manual_rewritten_utterance,
                            "auto": query.automatic_rewritten_utterance,
                        },
                    )
                    topic = Cast2020TopicRecord(
                        IDItem(query.query_id),
                        SimpleTextItem(query.raw_utterance),
                        decontextualized,
                        ConversationHistory(node.conversation(False)),
                    )

                    if topic_number == query.topic_number:
                        node = node.add(ConversationTreeNode(topic))
                    else:
                        conversation = []
                        node = ConversationTreeNode(topic)
                        topic_number = query.topic_number

                    records.append(topic)

                    conversation.append(node)
                    node = node.add(
                        ConversationTreeNode(
                            DocumentRecord.from_id(query.manual_canonical_result_id)
                        )
                    )
                    conversation.append(node)
            except Exception:
                logging.exception("Error while computing topic records")

            return records

    Topics.HANDLERS.update(
        {
            # _irds.trec_cast.Cast2019Query: Cast2019TopicsHandler,
            _irds.trec_cast.Cast2020Query: Cast2020TopicsHandler,
            # _irds.trec_cast.Cast2021Query: Cast2021TopicsHandler,
            # _irds.trec_cast.Cast2022Query: Cast2022TopicsHandler
        }
    )


class Adhoc(ir.Adhoc, IRDSId):
    pass


class AdhocRun(ir.AdhocRun, IRDSId):
    pass


class TrainingTriplets(ir.TrainingTriplets, IRDSId):
    """Training triplets from IR Dataset"""

    CONVERTERS = {
        GenericDocPair: lambda qid, doc1_id, doc2_id: (
            IDTopicRecord.from_id(qid),
            IDDocumentRecord.from_id(doc1_id),
            IDDocumentRecord.from_id(doc2_id),
        )
    }

    @cached_property
    def topic_recordtype(self) -> Type[Record]:
        """The set of records for topics"""
        return TopicRecord.from_types("TopicIDRecord", IDItem, module=__name__)

    @cached_property
    def document_recordtype(self) -> Type[Record]:
        """The class for documents"""
        return DocumentRecord.from_types("DocumentIDRecord", IDItem, module=__name__)

    @cached_property
    def converter(self):
        return TrainingTriplets.CONVERTERS[self.dataset.docpairs_cls()]

    def iter(
        self,
    ) -> Iterator[Tuple[ir.TopicRecord, ir.DocumentRecord, ir.DocumentRecord]]:
        ds = self.dataset

        logging.info("Starting to generate triplets")
        yield from (self.converter(*entry) for entry in ds.docpairs_iter())
        logging.info("Ending triplet generation")

    def count(self):
        """Returns the length or None"""
        return self.dataset.docpairs_count()



# See documentation on https://datamaestro.readthedocs.io

from collections import namedtuple
import gzip
import json
from pathlib import Path
from typing import Iterator, NamedTuple
import attrs
from datamaestro.definitions import datatasks, datatags, dataset
from datamaestro.download.single import filedownloader
from datamaestro.utils import HashCheck


from datamaestro_text.data.conversation.orconvqa import OrConvQADataset
from datamaestro.data.ml import Supervised

from datamaestro_text.data.ir import DocumentStore
from datamaestro_text.data.ir.formats import OrConvQADocument
from datamaestro_text.data.ir.stores import OrConvQADocumentStore
from datamaestro_text.datasets.irds.data import LZ4DocumentStore
from datamaestro_text.datasets.irds.helpers import lz4docstore_downloader


@datatags("conversation", "context", "query")
@datatasks("query rewriting")
@filedownloader(
    "train.jsonl",
    "https://ciir.cs.umass.edu/downloads/ORConvQA/preprocessed/train.txt",
    checker=HashCheck("7513a9ef12d8b7a4471166dc4fef77b7"),
)
@filedownloader(
    "dev.jsonl",
    "https://ciir.cs.umass.edu/downloads/ORConvQA/preprocessed/dev.txt",
    checker=HashCheck("7765658995cc9ffd5eb39a400d814b20"),
)
@filedownloader(
    "test.jsonl",
    "https://ciir.cs.umass.edu/downloads/ORConvQA/preprocessed/test.txt",
    checker=HashCheck("0cf3a755f06297b9c02e7db45f8dc8be"),
)
@dataset(
    Supervised,
    url="https://github.com/prdwb/orconvqa-release",
)
def preprocessed(train, dev, test):
    """Open-Retrieval Conversational Question Answering datasets

    OrConvQA is an aggregation of three existing datasets:

    1. the QuAC dataset that offers information-seeking conversations,
    1. the CANARD dataset that consists of context-independent rewrites of QuAC questions, and
    3. the Wikipedia corpus that serves as the knowledge source of answering questions.

    Each dataset is an instance of :class:`datamaestro_text.data.conversation.OrConvQADataset`
    """
    return {
        "train": OrConvQADataset(path=train),
        "validation": OrConvQADataset(path=dev),
        "test": OrConvQADataset(path=test),
    }


def orConvQADocumentReader(source: Path) -> Iterator[OrConvQADocumentStore.NAMED_TUPLE]:
    with gzip.open(source, "rt") as fp:
        for line in fp:
            yield OrConvQADocumentStore.NAMED_TUPLE(**json.loads(line))


@lz4docstore_downloader(
    "all_blocks",
    "https://ciir.cs.umass.edu/downloads/ORConvQA/all_blocks.txt.gz",
    orConvQADocumentReader,
    OrConvQADocumentStore.NAMED_TUPLE,
    "id",
    checker=HashCheck("1095a3408690e7bbe4d8a87a2bae6356"),
    size=5_086_902_800,
    count_hint=11_377_951,
)
@dataset(
    OrConvQADocumentStore,
    url="https://github.com/prdwb/orconvqa-release",
)
def passages(all_blocks):
    """orConvQA wikipedia files

    OrConvQA is an aggregation of three existing datasets:

    1. the QuAC dataset that offers information-seeking conversations,
    1. the CANARD dataset that consists of context-independent rewrites of QuAC questions, and
    3. the Wikipedia corpus that serves as the knowledge source of answering questions.
    """
    return {"path": all_blocks, "count": 11_377_951}



from functools import cached_property
from typing import Iterator, List, Optional
from attr import define
import json
from datamaestro.data import File
from datamaestro.record import recordtypes

from datamaestro_text.data.ir.base import (
    TopicRecord,
    GenericTopicRecord,
    IDItem,
    SimpleTextItem,
)


from .base import (
    AnswerConversationRecord,
    AnswerEntry,
    ConversationRecord,
    ConversationTree,
    RetrievedEntry,
    SimpleDecontextualizedItem,
    SingleConversationTree,
)
from . import ConversationDataset


@define(kw_only=True)
class OrConvQADatasetAnswer:
    text: staticmethod
    answer_start: 0
    bid: Optional[int] = None


@define(kw_only=True)
class OrConvQADatasetHistoryEntry:
    question: str
    """The question"""

    answer: OrConvQADatasetAnswer


@define(kw_only=True)
class OrConvQADatasetEntry:
    """A query with past history"""

    query_id: int
    """Question number"""

    query: str
    """The last issued query"""

    rewrite: str
    """Manually rewritten query"""

    history: List[OrConvQADatasetHistoryEntry]
    """The list of queries asked by the user"""

    answer: OrConvQADatasetAnswer

    evidences: List[str]
    """Evidence sources for the conversation"""

    retrieval_labels: List[int]
    """Relevance status for evidences"""


@recordtypes(SimpleDecontextualizedItem)
class OrConvQATopicRecord(GenericTopicRecord):
    pass


@recordtypes(AnswerEntry, RetrievedEntry)
class OrConvQAAnswerRecord(AnswerConversationRecord):
    pass


class OrConvQADataset(ConversationDataset, File):
    def entries(self) -> Iterator[OrConvQADatasetEntry]:
        """Iterates over re-written query with their context"""
        with self.path.open("rt") as fp:
            for line in fp.readlines():
                entry = json.loads(line)
                yield OrConvQADatasetEntry(
                    query_id=entry["qid"],
                    query=entry["question"],
                    evidences=entry["evidences"],
                    retrieval_labels=entry["retrieval_labels"],
                    rewrite=entry["rewrite"],
                    answer=OrConvQADatasetAnswer(**entry["answer"]),
                    history=[
                        OrConvQADatasetHistoryEntry(
                            question=history_entry["question"],
                            answer=OrConvQADatasetAnswer(**history_entry["answer"]),
                        )
                        for history_entry in entry["history"]
                    ],
                )

    def __iter__(self) -> Iterator[ConversationTree]:
        history: List[ConversationRecord] = []
        current_id: Optional[str] = None

        for entry in self.entries():
            # Creates a new conversation if needed
            cid, query_no = entry.query_id.rsplit("#", 1)
            if cid != current_id:
                if current_id is not None:
                    history.reverse()
                    yield SingleConversationTree(current_id, history)

                current_id = cid
                history = []

            # Add to current
            history.append(
                OrConvQATopicRecord(
                    IDItem(query_no),
                    SimpleTextItem(entry.query),
                    SimpleDecontextualizedItem(entry.rewrite),
                )
            )
            history.append(
                OrConvQAAnswerRecord(
                    AnswerEntry(entry.answer.text),
                    RetrievedEntry(entry.evidences, entry.retrieval_labels),
                )
            )

        # Yields the last one
        history.reverse()
        yield SingleConversationTree(current_id, history)



from pathlib import Path
from typing import Iterator, Tuple, Type

from experimaestro import Param, Option, Constant, Meta
from datamaestro.definitions import argument
from datamaestro.record import Record
import datamaestro_text.data.ir as ir
from datamaestro_text.data.ir.base import GenericTopicRecord, IDItem, SimpleTextItem
from datamaestro_text.interfaces.plaintext import read_tsv


@argument("path", type=Path)
@argument("separator", type=str, default="\t", ignored=True)
class AdhocRunWithText(ir.AdhocRun):
    "(qid, doc.id, query, passage)"
    pass


@argument("path", type=Path)
@argument("separator", type=str, default="\t", ignored=True)
class Topics(ir.Topics):
    "Pairs of query id - query using a separator"

    def iter(self):
        return (
            GenericTopicRecord(IDItem(qid), SimpleTextItem(title))
            for qid, title in read_tsv(self.path)
        )

    @property
    def topic_recordtype(self) -> Type[Record]:
        """The class for topics"""
        return GenericTopicRecord


class Documents(ir.Documents):
    "One line per document, format pid<SEP>text"

    path: Param[Path]
    separator: Meta[str] = "\t"

    pass



from .base import (
    AnswerEntry,
    ConversationDataset,
    ConversationHistory,
    ConversationHistoryItem,
    ConversationRecord,
    DecontextualizedItem,
    TopicConversationRecord,
    AnswerConversationRecord,
)
